package com.netease.alive_flutter_plugin

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.netease.nis.alivedetected.ActionType
import com.netease.nis.alivedetected.AliveDetector
import com.netease.nis.alivedetected.DetectedListener
import com.netease.nis.alivedetected.NISCameraPreview
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel.Result

/**
 * Created by hzhuqi on 2020/11/30
 */
class AliveHelper {
    private var aliveDetector = AliveDetector.getInstance()
    var events: EventChannel.EventSink? = null
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    fun init(context: Context, cameraPreview: NISCameraPreview, businessId: String, timeout: Int) {
        aliveDetector.init(context, cameraPreview, businessId)
        aliveDetector.setTimeOut(timeout.toLong() * 1000)
    }

    fun startDetect(result: Result) {
        val map = HashMap<String, Any?>()
        aliveDetector.startDetect()
        aliveDetector.setDetectedListener(object : DetectedListener {
            override fun onPassed(isPassed: Boolean, token: String?) {
                uiThreadHandler.post {
                    map["type"] = "checked"
                    map["message"] = token
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(AliveDetector.TAG, "Reply already submitted")
                    }
                }
            }

            override fun onCheck() {
                //本地验证通过提交远程检测
            }

            override fun onActionCommands(actionTypes: Array<ActionType>) {
                uiThreadHandler.post {
                    val actions = buildActionCommand(actionTypes)
                    map["actions"] = actions
                    try {
                        result.success(map)
                    } catch (e: Exception) {
                        Log.e(AliveDetector.TAG, "Reply already submitted")
                    }
                }
            }

            override fun onReady(p0: Boolean) {
            }

            override fun onError(code: Int, msg: String?, token: String?) {
                uiThreadHandler.post {
                    map["type"] = "exception"
                    map["message"] = msg
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(AliveDetector.TAG, "Reply already submitted")
                    }
                }
            }

            override fun onStateTipChanged(actionType: ActionType?, stateTip: String?) {
                uiThreadHandler.post {
                    map["type"] = "checking"
                    map["currentStep"] = actionType?.actionID?.toInt()
                    map["message"] = stateTip
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(AliveDetector.TAG, "Reply already submitted")
                    }
                }
            }

            override fun onOverTime() {
                uiThreadHandler.post {
                    map["type"] = "exception"
                    map["message"] = "timeout"
                    try {
                        events?.success(map)
                    } catch (e: Exception) {
                        Log.e(AliveDetector.TAG, "Reply already submitted")
                    }
                }
            }

        })
    }

    fun stopDetect(result: Result) {
        aliveDetector.stopDetect()
    }

    fun destroy() {
        aliveDetector.destroy()
    }

    private fun buildActionCommand(actionCommands: Array<ActionType>): String {
        val commands = StringBuilder()
        for (actionType in actionCommands) {
            commands.append(actionType.actionID)
        }
        return commands.toString()
    }

}