package com.netease.alive_flutter_plugin;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PaintFlagsDrawFilter;
import android.graphics.Path;
import android.graphics.Point;
import android.graphics.Region;
import android.os.Build;
import android.util.AttributeSet;

import com.netease.nis.alivedetected.NISCameraPreview;

/**
 * @author liuxiaoshuai
 * @date 2021/10/18
 * @desc
 * @email liulingfeng@mistong.com
 */
public class CircleCameraPreview extends NISCameraPreview {
    /**
     * 剪切路径
     */
    private Path clipPath;

    /**
     * 中心点坐标
     */
    private Point centerPoint;

    public CircleCameraPreview(Context context) {
        super(context);

        init();
    }

    public CircleCameraPreview(Context context, AttributeSet attrs) {
        super(context, attrs);

        init();
    }

    public CircleCameraPreview(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        init();
    }

    private void init() {
        this.setFocusable(true);
        this.setFocusableInTouchMode(true);
        clipPath = new Path();
        centerPoint = new Point();
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        // 坐标转换为实际像素
        int widthSize = MeasureSpec.getSize(widthMeasureSpec);
        int heightSize = MeasureSpec.getSize(heightMeasureSpec);
        // 计算出圆形的中心点
        centerPoint.x = widthSize >> 1;
        centerPoint.y = heightSize >> 1;
        // 计算出最短的边的一半作为半径
        /**
         * 半径
         */
        int radius = Math.min(centerPoint.x, centerPoint.y);
        clipPath.reset();
        clipPath.addCircle(centerPoint.x, centerPoint.y, radius, Path.Direction.CCW);
        setMeasuredDimension(widthSize, heightSize);
    }

    @Override
    public void draw(Canvas canvas) {
        //裁剪画布，并设置其填充方式
        canvas.setDrawFilter(new PaintFlagsDrawFilter(0, Paint.ANTI_ALIAS_FLAG | Paint.FILTER_BITMAP_FLAG));
        if (Build.VERSION.SDK_INT >= 26) {
            canvas.clipPath(clipPath);
        } else {
            canvas.clipPath(clipPath, Region.Op.REPLACE);
        }
        super.draw(canvas);
    }
}
