import 'package:alive_flutter_plugin/alive_flutter_plugin.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class LivePage extends StatefulWidget {
  @override
  _LivePageState createState() => _LivePageState();
}

class _LivePageState extends State<LivePage> {
  final AliveFlutterPlugin aliveFlutterPlugin = new AliveFlutterPlugin();

  String _result = "请开始活体检测";
  int _currentStep = 0;
  String action = "";

  var eventChannel = const EventChannel("yd_alive_flutter_event_channel");

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      init();
    });
    eventChannel.receiveBroadcastStream().listen(_onData, onError: _onError);
  }

  void _onData(response) {
    if (response is Map) {
      setState(() {
        var type = response["type"];
        if (type == "checking") {
          _currentStep = response["currentStep"];
          _result = response["message"];
        } else if (type == "checked") {
          _result = response["message"];
          _currentStep = 0;
          stopLive();
        } else if (type == "exception") {
          _result = response["message"];
        }
      });
    }
  }

  _onError(Object error) {
    PlatformException exception = error;
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('易盾活体检测'),
        ),
        body: new SingleChildScrollView(
          physics: BouncingScrollPhysics(),
          child: new Center(
            child: _buildContent(context),
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    aliveFlutterPlugin.stopLiveDetect();
    if (defaultTargetPlatform == TargetPlatform.android) {
      //android需要释放资源，否则会引发内存泄露
      aliveFlutterPlugin.destroy();
    }
    super.dispose();
  }

  void init() {
    aliveFlutterPlugin.init("易盾业务id", 40);
  }

  void startLive() {
    aliveFlutterPlugin.startLiveDetect().then((value) {
      /**
       * action表示返回的动作，动作状态表示：0——正面，1——右转，2——左转，3——张嘴，4——眨眼。
       */
      var actions = value["actions"];
      print(actions);
    });
  }

  void stopLive() {
    _currentStep = 0;
    aliveFlutterPlugin.stopLiveDetect();
  }

  Widget _buildContent(BuildContext context) {
    return Center(
      widthFactor: 2,
      child: new Column(
        children: [
          showDetailText(),
          showFaceImageWidget(),
          showAnimatedGif(),
          startDetectButton(),
          stopDetectButton(),
          backButton(context),
        ],
      ),
    );
  }

  Widget showDetailText() {
    return Center(
      child: Container(
        margin: EdgeInsets.fromLTRB(20, 20, 20, 0),
        color: Colors.white,
        child: Text(_result, textAlign: TextAlign.center),
        width: 300,
        height: 40,
      ),
    );
  }

  Widget showFaceImageWidget() {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return new Center(
        child: Container(
            width: 300,
            height: 300,
            child: AndroidView(
                viewType: 'platform-view-alive',
                creationParams: {"width": 300, "height": 400},
                creationParamsCodec: const StandardMessageCodec()),
            alignment: Alignment.center),
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      return new Center(
        child: Container(
          width: 200,
          height: 200,
          decoration: BoxDecoration(
              borderRadius: BorderRadius.all(Radius.circular(100))),
          child: UiKitView(
            viewType: 'com.flutter.alive.imageview',
            creationParams: {
              "width": "200",
              "height": "200",
              "radius": "100",
            },
            //参数的编码方式
            creationParamsCodec: const StandardMessageCodec(),
          ),
        ),
      );
    }
    return Text('$defaultTargetPlatform is not yet supported by this plugin');
  }

  Widget startDetectButton() {
    return new Container(
      child: SizedBox(
        child: new CustomButton(
          onPressed: () {
            startLive();
          },
          title: "开始检测",
        ),
        width: double.infinity,
        height: 49,
      ),
      margin: EdgeInsets.fromLTRB(40, 5, 40, 5),
    );
  }

  Widget stopDetectButton() {
    return new Container(
      child: SizedBox(
        child: new CustomButton(
          onPressed: () {
            stopLive();
          },
          title: "停止检测",
        ),
        width: double.infinity,
        height: 49,
      ),
      margin: EdgeInsets.fromLTRB(40, 5, 40, 5),
    );
  }

  Widget backButton(BuildContext context) {
    return new Container(
      child: SizedBox(
        child: new CustomButton(
          onPressed: () {
            Navigator.pop(context);
          },
          title: "返回",
        ),
        width: double.infinity,
        height: 49,
      ),
      margin: EdgeInsets.fromLTRB(40, 5, 40, 5),
    );
  }

  Widget showAnimatedGif() {
    /**
     * _currentStep == 1 右转
     * _currentStep == 2 左转
     * _currentStep == 3 张嘴
     * _currentStep == 4 眨眼
     */
    var image = '';
    if (_currentStep == 1) {
      image = 'images/turn-right.gif';
    } else if (_currentStep == 2) {
      image = 'images/turn-left.gif';
    } else if (_currentStep == 3) {
      image = 'images/open-mouth.gif';
    } else if (_currentStep == 4) {
      image = 'images/open-eyes.gif';
    } else {
      image = 'images/pic_front.png';
    }
    return new Container(
      child: Container(
        width: 120,
        height: 120,
        child: Image.asset(
          image,
        ),
      ),
    );
  }
}

/// 封装 按钮
class CustomButton extends StatelessWidget {
  final VoidCallback onPressed;
  final String title;

  const CustomButton({@required this.onPressed, this.title});

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return new FlatButton(
      onPressed: onPressed,
      child: new Text("$title"),
      color: Colors.blue,
      highlightColor: Colors.white,
      splashColor: Colors.white,
      textColor: Colors.white,
      padding: EdgeInsets.fromLTRB(10, 5, 10, 5),
      shape: RoundedRectangleBorder(
          side: BorderSide.none,
          borderRadius: BorderRadius.all(Radius.circular(10))),
    );
  }
}
